@@ -1,39 +1,54 @@
 package Biber::Constants;
-use v5.16;
+use v5.24;
 use strict;
 use warnings;
 
+use Encode;
 use Encode::Alias;
 
-use base 'Exporter';
+use parent qw(Exporter);
+use Biber::Date::Format;
+use Text::CSV;
 
 our @EXPORT = qw{
                   $CONFIG_DEFAULT_BIBER
-                  %CONFIG_DEFAULT_BIBLATEX
-                  %CONFIG_SCOPE_BIBLATEX
-                  %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS
-                  %NOSORT_TYPES
-                  %DM_DATATYPES
-                  %LOCALE_MAP
-                  %LOCALE_MAP_R
-                  %REMOTE_MAP
+                  $CONFIG_CSV_PARSER
                   $BIBER_CONF_NAME
                   $BCF_VERSION
                   $BBL_VERSION
                   $BIBER_SORT_FINAL
+                  $BIBER_SUPPRESS_FINAL
                   $BIBER_SORT_NULL
                   $LABEL_FINAL
+                  %CONFIG_DEFAULT_BIBLATEX
+                  %CONFIG_OPTSCOPE_BIBLATEX
+                  %CONFIG_SCOPEOPT_BIBLATEX
+                  %CONFIG_OPTTYPE_BIBLATEX
+                  %CONFIG_BIBLATEX_ENTRY_OPTIONS
+                  %CONFIG_BIBLATEX_NAMELIST_OPTIONS
+                  %CONFIG_BIBLATEX_NAME_OPTIONS
+                  %CONFIG_META_MARKERS
+                  %CONFIG_DATE_PARSERS
+                  %DATAFIELD_SETS
+                  %DM_DATATYPES
+                  %LOCALE_MAP
+                  %LOCALE_MAP_R
+                  %REMOTE_MAP
+                  %DS_EXTENSIONS
+                  %UNIQUENAME_CONTEXTS
+                  %UNIQUENAME_VALUES
+                  %MONTHS
               };
 
 # Version of biblatex control file which this release expects. Matched against version
 # passed in control file. Used when checking the .bcf
-our $BCF_VERSION = '2.6';
+our $BCF_VERSION = '3.5';
 # Format version of the .bbl. Used when writing the .bbl
-our $BBL_VERSION = '2.4';
+our $BBL_VERSION = '3.0';
 
 # Global flags needed for sorting
-our $BIBER_SORT_FINAL = 0;
-our $BIBER_SORT_NULL  = 0;
+our $BIBER_SORT_FINAL;
+our $BIBER_SORT_NULL;
 
 # the name of the Biber configuration file, which should be
 # either returned by kpsewhich or located at "$HOME/.$BIBER_CONF_NAME"
@@ -52,100 +67,152 @@ unless ($locale) {
   }
 }
 
-# nosort type category shortcuts
-our %NOSORT_TYPES = (
-                     type_name => {
-                                   author => 1,
-                                   afterword => 1,
-                                   annotator => 1,
-                                   bookauthor => 1,
-                                   commentator => 1,
-                                   editor => 1,
-                                   editora => 1,
-                                   editorb => 1,
-                                   editorc => 1,
-                                   foreword => 1,
-                                   holder => 1,
-                                   introduction => 1,
-                                   namea => 1,
-                                   nameb => 1,
-                                   namec => 1,
-                                   shortauthor => 1,
-                                   shorteditor => 1,
-                                   translator => 1
-                                  },
-                     type_title => {
-                                    booktitle => 1,
-                                    eventtitle => 1,
-                                    issuetitle => 1,
-                                    journaltitle => 1,
-                                    maintitle => 1,
-                                    origtitle => 1,
-                                    title => 1
-                                   }
-);
+our %MONTHS = ('jan' => '1',
+              'feb' => '2',
+              'mar' => '3',
+              'apr' => '4',
+              'may' => '5',
+              'jun' => '6',
+              'jul' => '7',
+              'aug' => '8',
+              'sep' => '9',
+              'oct' => '10',
+              'nov' => '11',
+              'dec' => '12');
+
+# datafieldsets
+our %DATAFIELD_SETS = ();
 
 # datatypes for data model validation
 our %DM_DATATYPES = (
-                     integer => qr/\A\d+\z/xms,
+                     integer  => qr/\A\d+\z/xms,
                      datepart => qr/\A\d+\z/xms
-);
+                    );
+
+# Mapping of data source and output types to extensions
+our %DS_EXTENSIONS = (
+                      bbl        => 'bbl',
+                      bblxml     => 'bblxml',
+                      bibtex     => 'bib',
+                      biblatexml => 'bltxml'
+                      );
+
+# Mapping of biblatex uniquename option to disambiguation level
+our %UNIQUENAME_CONTEXTS = (0 => 'none',        # false
+                            1 => 'init',        # init
+                            2 => 'initorfull',  # full/true
+                            3 => 'init',        # allinit
+                            4 => 'initorfull',  # allfull
+                            5 => 'init',        # mininit
+                            6 => 'initorfull'); # minfull
+
+# Mapping of strings to numeric uniquename values for easier biblatex processing
+our %UNIQUENAME_VALUES = ('none' => 0, 'init' => 1, full => '2');
 
 # Biber option defaults. Mostly not needed outside of tool mode since they are passed by .bcf
-
 our $CONFIG_DEFAULT_BIBER = {
-  clrmacros           => { content => 0 },
-  collate             => { content => 1 },
-  collate_options     => { option => { level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
-  graph               => { content => 0 },
-  debug               => { content => 0 },
-  decodecharsset      => { content => 'base' },
-  dot_include         => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
-  fastsort            => { content => 0 },
-  fixinits            => { content => 0 },
-  input_encoding      => { content => 'UTF-8' },
-  input_format        => { content => 'bibtex' },
-  listsep             => { content => 'and' },
-  mincrossrefs        => { content => 2 },
-  mssplit             => { content => '_' },
-  namesep             => { content => 'and' },
-  nodieonerror        => { content => 0 },
-  noinit              => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}/},
-                                       {value => q/[\x{2bf}\x{2018}]/} ] },
-  nolog               => { content => 0 },
-  nostdmacros         => { content => 0 },
-  nosort              => { option => [ { name => 'type_name', value => q/\A\p{L}{2}\p{Pd}/ },
-                                       { name => 'type_name', value => q/[\x{2bf}\x{2018}]/ } ] },
-  onlylog             => { content => 0 },
-  others_string       => { content => 'others' },
-  output_align        => { content => 1 },
-  output_encoding     => { content => 'UTF-8' },
-  output_format       => { content => 'bbl' },
-  output_indent       => { content => '2' },
-  output_fieldcase    => { content => 'upper' },
-  output_resolve      => { content => 0 },
-  output_safechars    => { content => 0 },
-  output_safecharsset => { content => 'base' },
-  quiet               => { content => 0 },
-  sortcase            => { content => 1 },
-  sortfirstinits      => { content => 0 },
-  sortupper           => { content => 1 },
-  tool                => { content => 0 },
-  trace               => { content => 0 },
-  validate_config     => { content => 0 },
-  validate_control    => { content => 0 },
-  validate_datamodel  => { content => 0 },
-  wraplines           => { content => 0 },
-  xsvsep              => { content => q/\s*,\s*/ },
+  annotation_marker                           => { content => q/+an/ },
+  clrmacros                                   => { content => 0 },
+  collate_options                             => { option => {level => 4, variable => 'non-ignorable', normalization => 'prenormalized' }},
+  graph                                       => { content => 0 },
+  debug                                       => { content => 0 },
+  dieondatamodel                              => { content => 0 },
+  decodecharsset                              => { content => 'base' },
+  dot_include                                 => { option => {section => 1, xdata => 1, crossref => 1, xref => 1 }},
+  fixinits                                    => { content => 0 },
+  input_encoding                              => { content => 'UTF-8' },
+  input_format                                => { content => 'bibtex' },
+  isbn10                                      => { content => 0 },
+  isbn13                                      => { content => 0 },
+  isbn_normalise                              => { content => 0 },
+  listsep                                     => { content => 'and' },
+  mincrossrefs                                => { content => 2 },
+  minxrefs                                    => { content => 2 },
+  named_annotation_marker                     => { content => q/:/ },
+  namesep                                     => { content => 'and' },
+  no_bblxml_schema                            => { content => 0 },
+  no_bltxml_schema                            => { content => 0 },
+  nodieonerror                                => { content => 0 },
+  noinit                                      => { option => [ {value => q/\b\p{Ll}{2}\p{Pd}/},
+                                                               {value => q/[\x{2bf}\x{2018}]/} ] },
+  nolabel                                     => { option => [ {value => q/[\p{P}\p{S}\p{C}]+/} ] },
+#  nolabelwidthcount                          => { option =>  }, # default is nothing
+  nolog                                       => { content => 0 },
+  nostdmacros                                 => { content => 0 },
+  nosort                                      => { option => [ { name => 'setnames', value => q/\A\p{L}{2}\p{Pd}/ },
+                                                               { name => 'setnames', value => q/[\x{2bf}\x{2018}]/ } ] },
+  onlylog                                     => { content => 0 },
+  others_string                               => { content => 'others' },
+  output_align                                => { content => 0 },
+  output_annotation_marker                    => { content => '+an' },
+  output_named_annotation_marker              => { content => ':' },
+  output_encoding                             => { content => 'UTF-8' },
+  output_field_order                          => { content => 'options,abstract,names,lists,dates' },
+  output_format                               => { content => 'bbl' },
+  output_indent                               => { content => '2' },
+  output_fieldcase                            => { content => 'upper' },
+  output_listsep                              => { content => 'and' },
+  output_namesep                              => { content => 'and' },
+  output_resolve_xdata                        => { content => 0 },
+  output_resolve_crossrefs                    => { content => 0 },
+  output_resolve_sets                         => { content => 0 },
+  output_safechars                            => { content => 0 },
+  output_safecharsset                         => { content => 'base' },
+  output_xnamesep                             => { content => '=' },
+  quiet                                       => { content => 0 },
+  noskipduplicates                            => { content => 0 },
+  sortdebug                                   => { content => 0 },
+  sortcase                                    => { content => 1 },
+  sortupper                                   => { content => 1 },
+  strip_comments                              => { content => 0 },
+  tool                                        => { content => 0 },
+  trace                                       => { content => 0 },
+  nouri_encode                                => { content => 0 },
+  validate_bblxml                             => { content => 0 },
+  validate_bltxml                             => { content => 0 },
+  validate_config                             => { content => 0 },
+  validate_control                            => { content => 0 },
+  validate_datamodel                          => { content => 0 },
+  wraplines                                   => { content => 0 },
+  xnamesep                                    => { content => '=' },
+  xsvsep                                      => { content => q/\s*,\s*/ },
 };
 
+# Set up some re-usable CSV parsers here for efficiency reasons
+our $CONFIG_CSV_PARSER = Text::CSV->new ( { binary           => 1,
+                                            allow_whitespace => 1,
+                                            always_quote     => 1  } );
+
+# Set up some re-usable Date parsers here for efficiency reasons
+# We need two as the missing component data is in these objects, not
+# in the DT objects returned by ->parse_datetime() and this data will
+# likely be different for start/end
+our %CONFIG_DATE_PARSERS = ('start' => Biber::Date::Format->new(),
+                            'end'   => Biber::Date::Format->new());
+
+our %CONFIG_META_MARKERS = ();
+
 # default global options for biblatex
 # Used to set:
 # * Some tool-mode defaults (as there is no .bcf and some biblatex options
 #   cannot be set in a biber config file)
 our %CONFIG_DEFAULT_BIBLATEX = (
-  sortscheme => 'none',
-);
+                                sortingtemplatename    => 'tool',
+                                useauthor     => 1,
+                                useeditor     => 1,
+                                usetranslator => 1,
+                                maxbibnames   => 100,
+                                maxitems      => 100,
+                                minbibnames   => 100,
+                                maxalphanames => 100,
+                                maxcitenames  => 100,
+                                maxsortnames  => 100,
+                                minalphanames => 100,
+                                mincitenames  => 100,
+                                minsortnames  => 100,
+                                minitems      => 100,
+                                useprefix     => 0
+                               );
 
 # Set up some encoding aliases to map \inputen{c,x} encoding names to Encode
 # It seems that inputen{c,x} has a different idea of nextstep than Encode
@@ -162,7 +229,7 @@ define_alias('x-ascii'        => 'ascii'); # Encode doesn't resolve this one by
 define_alias('lutf8'          => 'UTF-8'); # Luatex
 define_alias('utf8x'          => 'UTF-8'); # UCS (old)
 
-# Defines sensible defaults for setting sort locale (bcp47) from babel/polyglossia language names
+# maps between bcp47 lang/locales and babel/polyglossia language names
 our %LOCALE_MAP = (
                    'acadian'         => 'fr-CA',
                    'american'        => 'en-US',
@@ -187,7 +254,7 @@ our %LOCALE_MAP = (
                    'bulgarian'       => 'bg-BG',
                    'canadian'        => 'en-CA',
                    'canadien'        => 'fr-CA',
-                   'catalan'         => 'ca-ES',
+                   'catalan'         => 'ca-AD',
                    'coptic'          => 'cop',
                    'croatian'        => 'hr-HR',
                    'czech'           => 'cs-CZ',
@@ -220,7 +287,7 @@ our %LOCALE_MAP = (
                    'japanese'        => 'ja-JP',
                    'kannada'         => 'kn-IN',
                    'lao'             => 'lo-LA',
-                   'latin'           => 'sr-Latn',
+                   'latin'           => 'la-Latn',
                    'latvian'         => 'lv-LV',
                    'lithuanian'      => 'lt-LT',
                    'lowersorbian'    => 'dsb-DE',
@@ -236,6 +303,7 @@ our %LOCALE_MAP = (
                    'ngerman'         => 'de-DE',
                    'nko'             => 'ha-NG',
                    'norsk'           => 'nb-NO',
+                   'norwegian'       => 'nn-NO',
                    'nynorsk'         => 'nn-NO',
                    'occitan'         => 'oc-FR',
                    'piedmontese'     => 'pms-IT',
@@ -250,13 +318,16 @@ our %LOCALE_MAP = (
                    'samin'           => 'se-NO',
                    'sanskrit'        => 'sa-IN',
                    'scottish'        => 'gd-GB',
-                   'serbian'         => 'sr-Cyrl',
+                   'serbian'         => 'sr-Latn',
                    'serbianc'        => 'sr-Cyrl',
                    'slovak'          => 'sk-SK',
                    'slovene'         => 'sl-SI',
                    'slovenian'       => 'sl-SI',
                    'spanish'         => 'es-ES',
                    'swedish'         => 'sv-SE',
+                   'swiss'           => 'de-CH',
+                   'swissgerman'     => 'de-CH',
+                   'nswissgerman'    => 'de-CH',
                    'syriac'          => 'syc',
                    'tamil'           => 'ta-IN',
                    'telugu'          => 'te-IN',
@@ -267,7 +338,7 @@ our %LOCALE_MAP = (
                    'turkmen'         => 'tk-TM',
                    'ukrainian'       => 'uk-UA',
                    'urdu'            => 'ur-IN',
-                   'UKenglish'       => 'en-GB',
+                   'UKenglish'       => 'en-UK',
                    'uppersorbian'    => 'hsb-DE',
                    'USenglish'       => 'en-US',
                    'usorbian'        => 'hsb-DE',
@@ -293,6 +364,7 @@ our %LOCALE_MAP_R = (
                      'br'         => 'breton',
                      'br-FR'      => 'breton',
                      'ca'         => 'catalan',
+                     'ca-AD'      => 'catalan',
                      'ca-ES'      => 'catalan',
                      'cop'        => 'coptic',
                      'cs'         => 'czech',
@@ -304,6 +376,8 @@ our %LOCALE_MAP_R = (
                      'de-1996'    => 'ngerman',
                      'de-AT'      => 'austrian',
                      'de-AT-1996' => 'naustrian',
+                     'de-CH'      => 'swissgerman',
+                     'de-CH-1996' => 'nswissgerman',
                      'de-DE'      => 'german',
                      'de-DE-1996' => 'ngerman',
                      'dsb'        => 'lowersorbian',
@@ -316,7 +390,7 @@ our %LOCALE_MAP_R = (
                      'en'         => 'english',
                      'en-AU'      => 'australian',
                      'en-CA'      => 'canadian',
-                     'en-GB'      => 'UKenglish',
+                     'en-UK'      => 'UKenglish',
                      'en-GB'      => 'british',
                      'en-NZ'      => 'newzealand',
                      'en-US'      => 'USenglish',
@@ -439,90 +513,80 @@ our %LOCALE_MAP_R = (
                      'vi-VN'      => 'vietnamese',
                     );
 
-# Defines the scope of each of the BibLaTeX configuration options
-#
-# PRESORT is not a "real" biblatex option passed by biblatex. It is defined
-# by the biblatex \DeclarePresort macro and is stored in here as it
-# can be global/per-type or per-entry and therefore it's natural to store it here.
-our %CONFIG_SCOPE_BIBLATEX = (
-  alphaothers        => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  controlversion     => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  debug              => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  datamodel          => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  dataonly           => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  inheritance        => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  labelalpha         => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labelalphatemplate => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labelnamefield     => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labelnameform      => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labelnamelang      => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labelnamespec      => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labelnumber        => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labeltitle         => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labeltitlefield    => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labeltitleform     => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labeltitlelang     => {GLOBAL => 0, PER_TYPE => 0, PER_ENTRY => 1},
-  labeltitlespec     => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labeltitleyear     => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labeldate          => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  labeldatespec      => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  maxalphanames      => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  maxbibnames        => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  maxcitenames       => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  maxitems           => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  minalphanames      => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  minbibnames        => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  mincitenames       => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  minitems           => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  presort            => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  singletitle        => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  skipbib            => {GLOBAL => 0, PER_TYPE => 1, PER_ENTRY => 1},
-  skipbiblist        => {GLOBAL => 0, PER_TYPE => 1, PER_ENTRY => 1},
-  skiplab            => {GLOBAL => 0, PER_TYPE => 1, PER_ENTRY => 1},
-  sortalphaothers    => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 0},
-  sortexclusion      => {GLOBAL => 0, PER_TYPE => 1, PER_ENTRY => 0},
-  sorting            => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  sortlocale         => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  sortscheme         => {GLOBAL => 1, PER_TYPE => 0, PER_ENTRY => 0},
-  uniquelist         => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  uniquename         => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  useauthor          => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  useeditor          => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  useprefix          => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-  usetranslator      => {GLOBAL => 1, PER_TYPE => 1, PER_ENTRY => 1},
-);
+# Holds the scope of each of the BibLaTeX configuration options from the .bcf
+our %CONFIG_OPTSCOPE_BIBLATEX;
+# Holds the options in a particular scope
+our %CONFIG_SCOPEOPT_BIBLATEX;
+# Holds the datatype of an option at a particular scope
+our %CONFIG_OPTTYPE_BIBLATEX;
 
 # For per-entry options, what should be set when we find them and
 # what should be output to the .bbl for biblatex.
 # Basically, here we have to emulate relevant parts of biblatex's options processing
 # for local entry-specific options, note therefore the presence here of some
 # options like max/mincitenames which are not passed in the .bcf
-our %CONFIG_BIBLATEX_PER_ENTRY_OPTIONS =  (
-  dataonly       => {OUTPUT => 1, INPUT => {'skiplab' => 1,
-                                            'skipbiblist' => 1,
-                                            'uniquename' => 0,
-                                            'uniquelist' => 0}},
-  maxitems       => {OUTPUT => 1},
-  minitems       => {OUTPUT => 1},
-  maxbibnames    => {OUTPUT => 1},
-  minbibnames    => {OUTPUT => 1},
-  maxcitenames   => {OUTPUT => 1},
-  mincitenames   => {OUTPUT => 1},
-  maxalphanames  => {OUTPUT => 0},
-  minalphanames  => {OUTPUT => 0},
-  maxnames       => {OUTPUT => ['maxcitenames', 'maxbibnames'], INPUT  => ['maxcitenames', 'maxbibnames']},
-  minnames       => {OUTPUT => ['mincitenames', 'minbibnames'], INPUT  => ['mincitenames', 'minbibnames']},
-  presort        => {OUTPUT => 0},
-  skipbib        => {OUTPUT => 1},
-  skipbiblist    => {OUTPUT => 1},
-  skiplab        => {OUTPUT => 1},
-  uniquelist     => {OUTPUT => 0},
-  useauthor      => {OUTPUT => 1},
-  useeditor      => {OUTPUT => 1},
-  useprefix      => {OUTPUT => 1},
-  usetranslator  => {OUTPUT => 1},
-);
+our %CONFIG_BIBLATEX_ENTRY_OPTIONS =
+  (
+   dataonly          => {OUTPUT => 1,
+                         INPUT => {skiplab     => 1,
+                                   skipbiblist => 1,
+                                   uniquename  => 0,
+                                   uniquelist  => 0}
+                        },
+   maxitems          => {OUTPUT => 1},
+   minitems          => {OUTPUT => 1},
+   maxbibnames       => {OUTPUT => 1},
+   minbibnames       => {OUTPUT => 1},
+   maxcitenames      => {OUTPUT => 1},
+   mincitenames      => {OUTPUT => 1},
+   maxsortnames      => {OUTPUT => 1},
+   minsortnames      => {OUTPUT => 1},
+   maxalphanames     => {OUTPUT => 0},
+   minalphanames     => {OUTPUT => 0},
+   maxnames          => {OUTPUT => ['maxcitenames', 'maxbibnames', 'maxsortnames'],
+                         INPUT  => ['maxcitenames', 'maxbibnames', 'maxsortnames']},
+   minnames          => {OUTPUT => ['mincitenames', 'minbibnames', 'minsortnames'],
+                         INPUT  => ['mincitenames', 'minbibnames', 'minsortnames']},
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+   noinherit         => {OUTPUT => 0},
+   presort           => {OUTPUT => 0},
+   skipbib           => {OUTPUT => 1},
+   skipbiblist       => {OUTPUT => 1},
+   skiplab           => {OUTPUT => 1},
+   sortingnamekeytemplatename => {OUTPUT => 1},
+   uniquenametemplatename => {OUTPUT => 1},
+   labelalphanametemplatename => {OUTPUT => 1},
+   uniquelist        => {OUTPUT => 0},
+   useauthor         => {OUTPUT => 1},
+   useeditor         => {OUTPUT => 1},
+   useprefix         => {OUTPUT => 1},
+   usetranslator     => {OUTPUT => 1},
+  );
 
+our %CONFIG_BIBLATEX_NAMELIST_OPTIONS =
+  (
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+  );
+
+our %CONFIG_BIBLATEX_NAME_OPTIONS =
+  (
+   nametemplates     => {OUTPUT => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename'],
+                         INPUT  => ['sortingnamekeytemplatename',
+                                    'uniquenametemplatename',
+                                    'labelalphanametemplatename']},
+  );
 
 1;
 
@@ -548,7 +612,7 @@ L<https://github.com/plk/biber/issues>.
 
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009-2014 François Charette and Philip Kime, all rights reserved.
+Copyright 2009-2018 François Charette and Philip Kime, all rights reserved.
 
 This module is free software.  You can redistribute it and/or
 modify it under the terms of the Artistic License 2.0.
